/**
* swarm Version 1 by Peter Organa. Jan, 08, 2008
* Visit http://blog.organa.ca for documentation, updates and more free code.
*
*
* Copyright (c) 2008 Peter Organa
* 
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use,
* copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following
* conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*/
package ca.organa {
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.geom.Point;
	public class entity extends baseEntity {
		private const TURN_SPEED:Number = 0.08;
		private const MOVE_SPEED:Number = 4;
		private var _neighbour:baseEntity;
		private var _obstacle:baseEntity;
		private var _heading:Point
		private var myArrow:MovieClip;
		private const TOO_CLOSE:Number = 20
		private const TOO_FAR:Number = 30;
		private var halfPI:Number = Math.PI / 2;
		private var radians:Number;
		public function entity(setID:int) {
			size = 0;
			storeID = setID;
			moveSpeed = MOVE_SPEED;
			myArrow = getChildByName("internalArrow") as MovieClip;			
			radians = (myArrow.rotation / 180) * Math.PI;
			currentVectorX = Math.cos(radians);
			currentVectorY = Math.sin(radians);
		}
		public function callUpdate():void {
			var dxHeading:Number;
			var dyHeading:Number;
			var distHeading:Number;
			var vxHeading:Number;
			var vyHeading:Number;
			var dxClosest:Number;
			var dyClosest:Number;
			var distClosest:Number;
			var normalizeClosest:Number;
			var vxClosest:Number;
			var vyClosest:Number;
			var vxAverage:Number;
			var vyAverage:Number;
			var normalizerAverage:Number;
			var rotationPosition:Number = 0;	
			//
			dxHeading = heading.x - this.x;
			dyHeading = heading.y - this.y;
			distHeading = Math.sqrt(dxHeading * dxHeading + dyHeading * dyHeading);
			vxHeading = dxHeading / distHeading;
			vyHeading = dyHeading / distHeading;
			//
			dxClosest = closeEntity.x - this.x;
			dyClosest = closeEntity.y - this.y;
			normalizeClosest = Math.sqrt(dxClosest * dxClosest + dyClosest * dyClosest);
			distClosest = Math.sqrt(dxClosest * dxClosest + dyClosest * dyClosest) - closeEntity.size;
			var dxObstacle:Number = closeObstacle.x - this.x;
			var dyObstacle:Number = closeObstacle.y - this.y;
			var normalizeObstacle:Number = Math.sqrt(dxObstacle * dxObstacle + dyObstacle * dyObstacle);
			var distObstacle:Number = normalizeObstacle - closeObstacle.size;
			
			vxClosest = dxClosest / normalizeClosest;
			vyClosest = dyClosest / normalizeClosest;
			//
			if (distObstacle < TOO_CLOSE) {
				vxAverage = -(dxObstacle / normalizeObstacle);
				vyAverage = -(dyObstacle / normalizeObstacle);
				myArrow.gotoAndStop(3);
			}else if (distClosest > TOO_FAR) {
				vxAverage = vxHeading + vxClosest;
				vyAverage = vyHeading + vyClosest;
				myArrow.gotoAndStop(1);
			} else if ( distClosest < TOO_CLOSE) {
				vxAverage = -vxClosest;
				vyAverage = -vyClosest;
				myArrow.gotoAndStop(3);
			} else {
				vxAverage = vxHeading;
				vyAverage = vyHeading;
				myArrow.gotoAndStop(2);
			}
			normalizerAverage = Math.sqrt(vxAverage * vxAverage + vyAverage * vyAverage);
			vxAverage = vxAverage / normalizerAverage;
			vyAverage = vyAverage / normalizerAverage;
			//
			var crossProduct:Number = (currentVectorX * vyAverage - vxAverage * currentVectorY);
			var dotProduct:Number = (currentVectorX * vxAverage + currentVectorY * vyAverage);
			var angleSin:Number = crossProduct;
			var angleDifference:Number;
			if (dotProduct > 0) {
				angleDifference = Math.asin(angleSin);	
			} else {
				angleDifference = Math.PI - Math.asin(angleSin);
			}
			var newRadians:Number;
			//Faster way of doing an abs.
			angleDifference = (angleDifference < 0 ? -angleDifference : angleDifference);
			if (crossProduct > 0) {
				//Turn right
				radians = radians  +  angleDifference * TURN_SPEED;
			} else {
				//Turn left
				radians = radians  -  angleDifference * TURN_SPEED;
			}
			currentVectorX = Math.cos(radians);
			currentVectorY = Math.sin(radians);
			myArrow.rotation = (radians / Math.PI) * 180;
			moveObject();
		}
		public function get closeObstacle():baseEntity {
			return _obstacle;
		}
		public function set closeObstacle(value:baseEntity):void {
			_obstacle = value;
		}
		public function get closeEntity():baseEntity {
			return _neighbour;
		}
		public function set closeEntity(value:baseEntity):void {
			_neighbour = value;
		}
		public function get heading():Point {
			return _heading;
		}
		public function set heading(value:Point):void {
			_heading = value;
		}
	}
}